<?php

namespace FiloBlu\Refilo\Model\Indexer\Processor;

use Magento\Catalog\Api\Data\ProductSearchResultsInterface;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\Product\Attribute\Source\StatusFactory;
use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\App\ResourceConnection;

class Stock
{
    /**
     * @var int
     */
    private $totalPages = 0;
    /**
     * @var ProductRepository
     */
    private $productRepository;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;
    /**
     * @var Status
     */
    private $statusFactory;
    /**
     * @var \stdClass
     */
    private $_item;
    /**
     * @var ResourceConnection
     */
    private $connection;

    /**
     * Stock constructor.
     * @param ProductRepository $productRepository
     * @param StatusFactory $statusFactory
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param ResourceConnection $connection
     */
    public function __construct(
        ProductRepository $productRepository,
        StatusFactory $statusFactory,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        ResourceConnection $connection
    ) {
        $this->productRepository = $productRepository;
        $this->statusFactory = $statusFactory;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->connection = $connection;
    }

    /**
     * @param $store
     * @param $batch_size
     * @param $batch_sequence
     * @param bool $ids
     * @return mixed
     */
    public function getProducts($store, $batch_size, $batch_sequence, $ids = false)
    {
        $productCollection = $this->prepareProductCollection($store, $batch_size, $batch_sequence, $ids);
        return ($productCollection) ? $this->processProductCollection($productCollection) : false;
    }

    /**
     * @param $store
     * @param $batch_size
     * @param $batch_sequence
     * @param bool $ids
     * @return bool|ProductSearchResultsInterface
     */
    protected function prepareProductCollection($store, $batch_size, $batch_sequence, $ids = false)
    {
        $connection = $this->connection->getConnection();
        $inventoryStock = $connection->getTableName('inventory_stock');
        foreach ($connection->fetchAll("SELECT * FROM {$inventoryStock}",[],\Zend_Db::FETCH_ASSOC) as $stock){
            $stockId = $stock['stock_id'];
            $inventoryPerStock = $connection->getTableName("inventory_stock_{$stockId}");
            $connection->fetchAll("SELECT `product_id` as `_id`,  FROM  {$inventoryPerStock}",[],\Zend_Db::FETCH_OBJ);
        };
//        if (($this->totalPages < $batch_sequence) && $this->totalPages != 0) {
//            return false;
//        }
//        $status = $this->statusFactory->create()->getVisibleStatusIds();
//        $this->searchCriteriaBuilder
//            ->addFilter('status', $status, 'in')
//            ->addFilter('store_id', $store, 'eq');
//
//        if (is_array($ids)) {
//            $this->searchCriteriaBuilder->addFilter('entity_id', $ids, 'in');
//        }
//
//        $searchCriteriaBuilder =  $this->searchCriteriaBuilder->create();
//
//        $searchCriteriaBuilder->setPageSize($batch_size);
//        $searchCriteriaBuilder->setCurrentPage($batch_sequence);
//        printf("Prepare Collection\n\r");
//        $collection = $this->productRepository->getList($searchCriteriaBuilder);
//
//        if (!$this->totalPages) {
//            $count = $collection->getTotalCount();
//            $this->totalPages = ceil($count / $batch_size);
//        }
//        printf("Page %d of %d \n\r", $batch_sequence, $this->totalPages);
//        return $collection;


    }

    /**
     * @param $_product_collection
     * @return array
     */
    protected function processProductCollection($_product_collection)
    {
        $this->_item = new \stdClass();
        $items = [];
        foreach ($_product_collection->getItems() as $product) {
            $this->_item->_id = (int) $product->getId();
            $this->_item->inventory = new \stdClass();
            $this->_item->inventory->in_stock = $product->isAvailable();
            $this->_item->inventory->backorders = 0; //ToDo fix it when magento will fix it
            $this->_item->inventory->saleable = $product->isSaleable();
            $item = $this->_item;
            $items[] = $item;
        }
        $this->productRepository->cleanCache();
        return $items;
    }
}
