<?php

namespace FiloBlu\Refilo\Model\Indexer;

use FiloBlu\Refilo\Helper\Data as HelperData;
use FiloBlu\Refilo\Model\Adapter\Mongo;
use FiloBlu\Refilo\Model\Indexer\Processor\Stock as ProcessorStock;
use Magento\Store\Model\StoreManagerInterface;

class Stock implements \Magento\Framework\Indexer\ActionInterface, \Magento\Framework\Mview\ActionInterface
{

    /**
     * @var Mongo
     */
    private $connection;

    /**
     * @var FiloBlu\Refilo\Helper\Data
     */
    private $helperData;
    /**
     * @var StoreManagerInterface
     */
    private $_storeManager;
    /**
     * @var int
     */
    private $batchSize;
    /**
     * @var int
     */
    private $batchSequence;
    /**
     * @var ProcessorStock
     */
    private $processorStock;

    /**
     * Stock constructor.
     * @param StoreManagerInterface $storeManager
     * @param ProcessorStock $processorStock
     * @param Mongo $connection
     * @param HelperData $helperData
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        ProcessorStock $processorStock,
        Mongo $connection,
        HelperData $helperData
    ) {
        $this->_storeManager = $storeManager;
        $this->processorStock = $processorStock;
        $this->connection = $connection;
        $this->helperData = $helperData;
        $this->batchSize = 20; //($batch_size = (int) $this->helperData->getConfigValue("sync/data/batch_size")) ? $batch_size : 100;
        $this->batchSequence = 1;
    }

    public function execute($ids)
    {
        $this->executeGeneric($ids);
    }

    public function executeFull()
    {
        $this->executeGeneric();
    }

    public function executeList(array $ids)
    {
        $this->executeGeneric($ids);
    }

    public function executeRow($id)
    {
        $this->executeGeneric([$id]);
    }

    protected function executeGeneric($ids = false)
    {
        $storeDataList = $this->_storeManager->getStores();
        echo memory_get_usage() / 1024 / 1024 . "\n"; // 36640

        foreach ($storeDataList as $key => $store) {
            $mtime = microtime();
            $mtime = explode(" ", $mtime);
            $mtime = $mtime[1] + $mtime[0];
            $starttime = $mtime;
            printf("Prepare for store %s", $store->getCode() . "\n\r");
            while ($productItems = $this->processorStock->getProducts($store->getId(), $this->batchSize, $this->batchSequence, $ids)) {
                echo memory_get_usage()  / 1024 / 1024 . "\n"; // 36640
                $mongo_collection = "catalog.products_test"; //. $store->getCode();
                $this->connection->update("_id", $productItems, $mongo_collection);
                $this->batchSequence++;
            }
            $this->processorStock->setTotalPages(0);
            $this->batchSequence = 1;
            $mtime = microtime();
            $mtime = explode(" ", $mtime);
            $mtime = $mtime[1] + $mtime[0];
            $endtime = $mtime;
            $totaltime = ($endtime - $starttime);
            echo memory_get_usage()  / 1024 / 1024 . "\n"; // 36640
            echo "Executed in " . $totaltime . " seconds";
        }
    }
}
